classdef Message
%Message A container for outgoing TraCI messages.

%   Copyright 2019 Universidad Nacional de Colombia,
%   Politecnico Jaime Isaza Cadavid.
%   Authors: Andres Acosta, Jairo Espinosa, Jorge Espinosa.
%   $Id: Message.m 48 2018-12-26 15:35:20Z afacostag $

	properties
		string
		queue
	end
end
